#include "Tribes.h"

Bitmap *IFF_hfoe, *IFF_hfriend;
Font *IFF_font;
bool IFF_loaded = false;
int IFF_lastIgnoreTargets;
bool IFF_wasDemocam = false;

void IFF_Load( ) {
	IFF_hfoe = (Bitmap *)Tribes::GetResource( "h_foe.bmp" )->data;
	IFF_hfriend = (Bitmap *)Tribes::GetResource( "h_friend.bmp" )->data;
	IFF_font = (Font *)Tribes::GetResource( "sf_white_7.pft" )->data;
	IFF_lastIgnoreTargets = Tribes::GetIgnoreTargets( );
	IFF_loaded = true;
}

void Beginframe_IFF( GraphicsAdapter *gfx ) {
	if ( !IFF_loaded )
		IFF_Load( );

	int mgrid, friendly_team;
	bool is_obs, is_third, is_playing, is_dead, is_democam;
	Entity *psc, *observed, *guicam;
	ClientData *cl;
	PlayerPSC *playerpsc;
	Vertex3f eye_position;
	CastRayOddball cro;

	// set the clipping area to the full screen
	if ( !Tribes::SetFullscreenClipping( gfx, true ) )
		return;

	// camera position is stored in the play gui for some reason? :-(
	PlayGUI *playGui = (PlayGUI *)Tribes::ObjectToAddress( "playGui" );
	if ( !playGui )
		return;

	
	SimTerrain *terrain = (SimTerrain *)Tribes::ObjectToAddress( 8 );
	if ( !terrain )
		return;

	mgrid = Tribes::GetManagerID( );
	if ( mgrid <= 0 )
		return;

	playerpsc = Tribes::GetPlayerPSC( );
	psc = Tribes::GetParentObject( );
	observed = Tribes::GetControlledObject( );
	guicam = playGui->camera;

	// observed will exist if psc does
	if ( !playerpsc || !psc )
		return;
	
	cl = Tribes::GetClientData( mgrid );
	is_obs = ( psc != observed );
	is_third = ( playerpsc->FirstPerson == 0 );
	is_democam = ( (PlayerPSC *)guicam != playerpsc );
	is_playing = ( cl->team != -1 );
	is_dead = ( cl->playerdata == NULL );

	if ( is_playing && !is_third && !is_obs && !is_democam ) {
		return;
	}

	if ( !is_playing || is_obs ) {
		friendly_team = ( observed != psc ) ? observed->team : 0;
	} else {
		friendly_team = ( psc->team );
	}

	/* get initial ignore targets */
	if ( is_democam && !IFF_wasDemocam ) {
		IFF_lastIgnoreTargets = Tribes::GetIgnoreTargets( );
		Tribes::IgnoreTargets( true );
	} else if ( !is_democam && IFF_wasDemocam ) {
		Tribes::IgnoreTargets( IFF_lastIgnoreTargets );
	}

	IFF_wasDemocam = is_democam;
	

	Tribes::CastRayInit( &cro );
	for ( cl = Tribes::GetClientFirst( ); cl != NULL; cl = cl->next ) {
		if ( ( cl->team == -1 ) || ( cl->playerdata == NULL ) )
			continue;

		if ( !is_democam && ( cl->playerdata == observed ) )
			continue;

		Entity *pl = cl->playerdata;
		Bitmap *icon = NULL;
		Vertex3f rotated_position, distance_position;
		Vertex2i screen;		

		if ( is_playing && !is_democam ) {
			if ( !Tribes::IsOnSensors( cl->id ) )
				continue;

			// this will keep us from seeing too far from the death cam
			if ( is_dead ) {			
				vectorSub( &pl->position, &eye_position, &distance_position );
				if ( vectorLength( &distance_position ) > ( terrain->visible_distance / 2 ) )
					continue;
			}
		}

		vectorAdd( &pl->position_back_left, &pl->position_top_right, &rotated_position );
		vectorMul( &rotated_position, &rotated_position, 0.5f );
		rotated_position.z += ( pl->PlayerShapeData->boxNormalHeight ) / 2;

		if ( !Tribes::CastRay( &playGui->camera_info->cam_matrix_w, &rotated_position, psc->id, &cro ) )
			continue;

		vectorTransform( &playGui->camera_info->cam_matrix_x, &rotated_position, &rotated_position );

		if ( Tribes::GetScreenCoordinates( &rotated_position, &screen, RAD2DEG( playGui->fov_rad_div_2 * 2 ) ) ) {
			icon = ( pl->team == friendly_team ) ? IFF_hfriend : IFF_hfoe;
			screen.x -= ( icon->width >> 1 );
			screen.y -= ( icon->height );

			Tribes::DrawBitmap( gfx, icon, screen.x, screen.y );

			if ( !is_playing || is_democam ) {
				char dist[128];
				float damage = 1.0f - ( cl->playerdata->damage / cl->playerdata->PlayerShapeData->shapeBase.maxDamage );
				Tribes::Sprintf( dist, "%s %.0fm %.0f%%", cl->name, vectorLength( &rotated_position ), damage * 100 );
				Tribes::DrawString( gfx, IFF_font, dist, screen.x, screen.y - icon->height - 2);
			}
		}
	}

	Tribes::CastRayFinish( &cro );
}